function output = H_methods_IT ( data, config, waitbar )

% Copyright 2012 Guiomar Niso, Ricardo Bruna

% This file is part of HERMES.
% 
% HERMES is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% HERMES is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with HERMES. If not, see <http://www.gnu.org/licenses/>.


% Checks the completitude of the configuration structure.
% if ~isfield ( config, 'measures' ), config.measures = {};        end
% if ~isfield ( config, 'window' ),   config.window   = struct (); end
% 
% if ~isfield ( config.window, 'length' ),    config.window.length    = size ( data, 1 ); end
% if ~isfield ( config.window, 'overlap' ),   config.window.overlap   = 0;                end
% if ~isfield ( config.window, 'alignment' ), config.window.alignment = 'epoch';          end
% if ~isfield ( config.window, 'fs' ),        config.window.fs        = 256;              end
% if ~isfield ( config.window, 'baseline' ),  config.window.baseline  = 0;                end



% Checks the existence of the waitbar.
if nargin < 3, waitbar = []; end

% Fetchs option data
MI_on  = H_check ( config.measures, 'MI'  );
PMI_on = H_check ( config.measures, 'PMI' );
TE_on  = H_check ( config.measures, 'TE'  );
PTE_on = H_check ( config.measures, 'PTE' );

% Windows the data.
data = H_window ( data, config.window );

% Gets the size of the analysis data.
[ samples, channels, windows, trials ] = size ( data );

% Nlags     = channels-1; % Los lags por def son 0
Nlags     = 1; % para ahorrar tiempo, de momento
Npart     = channels-2; % para la parcializacion
% tWindRad  = 10; % para la TMI

% Reservamos memoria para cada indice.
if MI_on,  output.MI.data  = zeros ( channels, channels, windows ); end
if PMI_on, output.PMI.data = zeros ( channels, channels, windows ); end
if TE_on,  output.TE.data  = zeros ( channels, channels, windows ); end
if PTE_on, output.PTE.data = zeros ( channels, channels, windows ); end

% Throttled variables.
interval_between_checks = 0.1;
tic

%  Recorremos todas las ventanas y todos los pares de sensores.
for window = 1: windows
    for ch1 = 1: channels
        for ch2 = ch1 + 1: channels
            
            data_ch1 = squeeze ( data ( :, ch1, window, : ) );
            data_ch2 = squeeze ( data ( :, ch2, window, : ) );
            
            % Places each trial in a separate cell field.
            if trials > 1
                celldata_ch1 = squeeze ( mat2cell ( permute ( data_ch1, [ 3 1 2 ] ), 1, samples, ones ( trials, 1 ) ) )';
                celldata_ch2 = squeeze ( mat2cell ( permute ( data_ch2, [ 3 1 2 ] ), 1, samples, ones ( trials, 1 ) ) )';
            else
                celldata_ch1 = { data_ch1' };
                celldata_ch2 = { data_ch2' };
            end
            
            if MI_on
                output.MI.data ( ch1, ch2, window ) = max ( mutual_information ( celldata_ch1, celldata_ch2, 'yLag', 0: Nlags - 1, 'k', config.Nneighbours ) );
                output.MI.data ( ch2, ch1, window ) = output.MI.data ( ch1, ch2, window );
            end
            
%             if H_check ( config.measures, 'TMI' )
%                 output.TMI.data ( i, j, window ) = max(mutual_information_t ( celldata_i, celldata_j, tWindRad,'yLag',0:Nlags-1,'k',config.Nneighbours));
%                 % asimetrica? output.TMI.data ( j, i, window ) = output.MI.data ( i, j, window );
%             end
            
            % Throttled check.
            if toc > interval_between_checks
                
                % Checks for user cancelation.
                if ( H_stop ), return, end
            end
            
            if PMI_on
                
                PartNodes = setdiff ( 1: channels, [ ch1, ch2 ] );
                Groups    = nchoosek ( PartNodes, Npart );
                
                for n3 = 1: size ( Groups, 1 )
                    PMI_origen = zeros ( 1, Nlags );
                    
                    data_ch3 = squeeze ( data ( :, Groups ( n3 ), window, : ) );
                    
                    if trials > 1,     celldata_ch3 = squeeze ( mat2cell ( data_ch3, samples, ones ( trials, 1 ) ) );
                    else               celldata_ch3 = { data_ch3 };   end
                    
                    for lag = 0: Nlags - 1
                        PMI_origen ( lag + 1 ) = min ( mutual_information_p ( celldata_ch1, celldata_ch2, celldata_ch3, 'yLag', lag, 'zLag', 0: Nlags - 1, 'k', config.Nneighbours ) );
                    end
                    output.PMI.data ( ch1, ch2, window ) = max ( PMI_origen );
                end
            end
            
            % Throttled check.
            if toc > interval_between_checks
                
                % Checks for user cancelation.
                if ( H_stop ), return, end
            end
            
            % This loop only is executed if some TE index is required.
            if TE_on || PTE_on
                
%                 for dt = 2: config.TimeDelay %%% Crazy, for the moment!!!!!
                for dt = config.TimeDelay
                    
                    w = delay_embed_future ( celldata_ch1, dt );
                    
                    % Places each trial in a separate cell field.
%                     if trials > 1,     w = mat2cell ( w, size ( w, 1 ), ones ( size ( w, 2 ), 1 ) );
%                     else               w = { w };   end
                    
                    if TE_on
                        output.TE.data ( ch1, ch2, window ) = max ( transfer_entropy ( celldata_ch1, celldata_ch2, w, 'yLag', 0: Nlags - 1, 'k', config.Nneighbours ) );
                        output.TE.data ( ch2, ch1, window ) = max ( transfer_entropy ( celldata_ch2, celldata_ch1, w, 'yLag', 0: Nlags - 1, 'k', config.Nneighbours ) );
                    end
                    
%                     if H_check ( config.measures, 'TTE' )
%                         output.TTE.data ( i, j, window ) = max(transfer_entropy_t ( celldata_i, celldata_j, w, tWindRad,'yLag',0:Nlags-1,'k',config.Nneighbours));
%                         output.TTE.data ( j, i, window ) = max(transfer_entropy_t ( celldata_j, celldata_i, w, tWindRad,'yLag',0:Nlags-1,'k',config.Nneighbours));
%                     end
                    
                    if PTE_on
                        
                        PartNodes = setdiff ( 1: channels, [ ch1, ch2 ] );
                        Groups    = nchoosek ( PartNodes, Npart );
                        
                        for n3 = 1: size ( Groups, 1 )
                            PTE_origen   = zeros ( 2, Nlags );
                            data_ch3     = squeeze ( data ( :, Groups ( n3 ), window, : ) );
                            
                            % Places each trial in a separate cell field.
                            if trials > 1,     celldata_ch3 = squeeze ( mat2cell ( data_ch3, samples, ones ( trials, 1 ) ) );
                            else               celldata_ch3 = { data_ch3 };     end
                            
                            for lag = 0: Nlags-1
                                PTE_origen ( 1, lag + 1 ) = min ( mutual_information_p ( celldata_ch1, celldata_ch2, celldata_ch3, 'yLag', lag, 'zLag', 0: Nlags - 1, 'k', config.Nneighbours ) );
                                PTE_origen ( 2, lag + 1 ) = min ( mutual_information_p ( celldata_ch2, celldata_ch1, celldata_ch3, 'yLag', lag, 'zLag', 0: Nlags - 1, 'k', config.Nneighbours ) );
                            end
                            
                            output.PTE.data ( ch1, ch2, window ) = max ( PTE_origen ( 1, : ) );
                            output.PTE.data ( ch2, ch1, window ) = max ( PTE_origen ( 2, : ) );
                        end
                    end
                end
            end
            
            % Throttled check.
            if toc > interval_between_checks
                tic
                
                % Checks for user cancelation.
                if ( H_stop ), return, end
                
                % Updates the waitbar.
                if ~isempty ( waitbar )
                    waitbar.progress ( 5:6 ) =  [ window windows ];
                    waitbar.progress ( 7 )   = ( 2 * channels - ch1 ) * ( ch1 - 1 ) / 2 + ( ch2 - ch1 );
                    waitbar.progress ( 8 )   = channels * ( channels - 1 ) / 2;
                    waitbar                  = H_waitbar ( waitbar );
                end
            end
        end
    end
    
    % Updates the waitbar.
    if ~isempty ( waitbar )
        waitbar.progress ( 5:6 )    =  [ window windows ];
        waitbar.progress ( 7: end ) = [];
        waitbar                     = H_waitbar ( waitbar );
    end
end
