function sizes = H_size ( data, dimensions )
%H_SIZE   Extension to Matlab SIZE function.

% Infinite indexing is only valid with linear dimensions vectors.
if any ( isinf ( dimensions (:) ) ) && sum ( size ( dimensions ) > 1 ) > 1, error ( 'Infinite indexing is valid with linear dimension inputs only.' ); end

% dimensions vector must be integer.
if any ( dimensions (:) ~= round ( dimensions (:) ) ), error ( 'Dimensions vector must be formed by integers or infinites.' ); end
if any ( isnan ( dimensions (:) ) ),                   error ( 'Dimensions vector must be formed by integers or infinites.' ); end

% If there is infinite indexing, reconstruct the vector.
if any ( isinf ( dimensions (:) ) )
    
    % Creates the cell to perform the expansion.
    expanded = cell ( size ( dimensions ) );
    
    % Goes through all the values.
    for value = 1: numel ( expanded )
        
        % The sucession [ x inf ] means [ x: ndims ].
        if isinf ( dimensions ( value ) )
            
            % If the infinite is in the begining of the vector, takes [ 1 inf ].
            if value == 1, start = 1;
            else           start = dimensions ( value - 1 ) + 1;
            end
            
            % Constructs [ x: ndims ].
            vector = start: ndims ( data );
            
            % Reshapes the vectors to match the original aligniation.
            shape = ones ( 1, max ( ndims ( vector ), find ( size ( dimensions ) > 1 ) ) );
            shape ( size ( dimensions ) > 1 ) = numel ( vector );
            
            expanded { value } = reshape ( vector, shape );
            
        % If the element is finite, continues.
        else
            expanded { value } = dimensions ( value );
        end
    end
    
    % Concatenates all the expanded vectors.
    dimensions = cell2mat ( expanded );
end

% Constructs the sizes vector with the same size as the dimensions vector.
sizes = zeros ( size ( dimensions ) );

% Fills the sizes version with each dimension size.
for dimension = 1: numel ( dimensions )
    
    % If dimension is 0 or negative, returns size 1.
    if dimensions ( dimension ) < 1
        sizes ( dimension ) = 1;
        
    % Otherwise returns the dimension size.
    else
        sizes ( dimension ) = size ( data, dimensions ( dimension ) );
        
    end
end