function pvalues = myranksum ( set1, set2, waitbar )

% Gets the mirrored matrices.
mir1 = permute ( set1, [ 1 3 2 4 5 ] );
mir2 = permute ( set2, [ 1 3 2 4 5 ] );
symm = all ( set1 (:) == mir1 (:) ) && all ( set2 (:) == mir2 (:) );

% If the matrices are symmetrical checks only the upper triangular.
if symm
    template    = triu   ( true ( H_size ( set1, [ 2 3 ] ) ), 1 );
    triangular  = repmat ( template, H_size ( set1, [ 0 0 4 inf ] ) );
    comparisons = find ( triangular );
    
% Otherwise checks the whole matrix.
else
    comparisons = 1: prod ( H_size ( set1, [ 2 inf ] ) );
end

% Throttled variables.
tinv = 0.2;
tic;

% Reserves memory for the output data.
pvalues = zeros ( H_size ( set1, [ 2 inf ] ) );

% Goes through all the comparisons.
for comparison = 1: numel ( comparisons )
    
    % Calculates the p-value associated to this comparison.
    pvalues ( comparisons ( comparison ) ) = ranksum ( set1 ( :, comparisons ( comparison ) ), set2 ( :, comparisons ( comparison ) ) );
    
    % Throttled check.
    if toc > tinv
        
        % Checks for user cancelation.
        if ( H_stop ), return, end
        
        % Updates the waitbar.
        if nargin == 3
            waitbar.progress (1) = comparison;
            waitbar.progress (2) = numel ( comparisons );
            waitbar = H_waitbar ( waitbar );
        end
        
        tic
    end
end

% If the matrices are symmetrical fills the lower triangular.
if symm
    pvalues = pvalues + permute ( pvalues, [ 2 1 3: ndims( pvalues ) ] );
    pvalues = pvalues + ( triangular + permute ( triangular, [ 2 1 3: ndims( triangular ) ] ) == 0 );
end
